/*
 * cram-md5.c
 *
 * This code was adapted from GNU Anubis, version 3.6.2
 * Copyright (C) 2001, 2002 The Anubis Team.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>
#include <sys/types.h>
#include <openssl/md5.h>

#include "merror.h"
#include "xmalloc.h"


/*
 * cram_md5()
 *
 * Writes MD5 digest of 'secret', 'challenge' to 'digest', which must be
 * 16 unsigned chars long. 'secret' and 'challenge' must not be NULL!
 */

void cram_md5(char *secret, size_t secret_len, 
	char *challenge, size_t challenge_len,
	unsigned char *digest)
{
    MD5_CTX context;
    unsigned char ipad[64];
    unsigned char opad[64];
    int i;
    
    memset(digest, 0, (size_t)16);
    memset(ipad, 0, sizeof(ipad));
    memset(opad, 0, sizeof(opad));
    
    if (secret_len > 64) 
    {
	MD5_Init(&context);
	MD5_Update(&context, (unsigned char *)secret, secret_len);
	MD5_Final(ipad, &context);
	MD5_Final(opad, &context);
    }
    else 
    {
	memcpy(ipad, secret, secret_len);
	memcpy(opad, secret, secret_len);
    }
    
    for (i = 0; i < 64; i++)
    {
	ipad[i] ^= 0x36;
	opad[i] ^= 0x5c;
    }
    
    MD5_Init(&context);
    MD5_Update(&context, ipad, (size_t)64);
    MD5_Update(&context, (unsigned char *)challenge, challenge_len);
    MD5_Final(digest, &context);
    
    MD5_Init(&context);
    MD5_Update(&context, opad, (size_t)64);
    MD5_Update(&context, digest, (size_t)16);
    MD5_Final(digest, &context);
    
    return;
}
