/*
 * xmalloc.h
 *
 * This file is part of the msmtp package, a very simple programm that 
 * works as an "SMTP plugin" for Mutt and possibly other MUAs.
 *
 * Copyright (C) 2000, 2003	Martin Lambers <marlam@web.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef XMALLOC_H
#define XMALLOC_H

#include <stdlib.h>
#include <stdarg.h>


/*
 * These functions are replacements for malloc(), calloc(), realloc()
 * that always return valid pointers, never NULL.
 * If there's not enough memory available, they die() instead.
 */

void *xmalloc(size_t size);
void *xcalloc(size_t n, size_t size);
void *xrealloc(void *ptr, size_t newsize);

/*
 * Replacement for strdup that always succeeds.
 */

char *xstrdup(const char *s);

/*
 * Replacement for vasprintf/asprtinf that always succeed.
 */

int xvasprintf(char **strp, const char *format, va_list ap);
int xasprintf(char **strp, const char *format, ...);

#endif
